#ifndef GLTRACKBALL_H
#define GLTRACKBALL_H

#include "../gmMath/gm.h"

class GLTrackball
{
public:
	// gets the single trackball instance
	static GLTrackball* getTrackball();

  // destroys the single trackball instance
  static void destroyTrackball();

	// moves the trackball transormation to the OpenGL matrix stack 
	void setGLView();

  // moves the trackball
  gmQuaternion move(float p1x, float p1y, float p2x, float p2y);

	// transforms the window coordinates and calls "move"
  // matches the glut MouseMove callback function
	void mouseMove(int x, int y);

  // converts the quaternion to a rotation matrix
	void ToRotationMatrix (gmMatrix4 &R) const;

	// gets the current orientation of the trackball
	gmQuaternion getQuaternion() const { return m_result; }

	// sets the window dimensions
	void setWindowDimensions(int w, int h);

	// applies a mouse click
	void mouseButton(int button, int state, int x, int y);
	
  // resets the trackball
	void reset();

  //sets the rotational part of the view transform specified in OpenGl
  void setViewTransform();
	
private:
	// constrcutor/destructor
	GLTrackball();
	~GLTrackball();

	// member variables
	static GLTrackball* m_pInstance;

	// projects the screen coordinates onto a sphere
	float m_projectToSphere(float x, float y);

  //store the view transform to compute correct trackball coordinates
  //use only the rotational part of the view transform
  gmMatrix3 viewTransform;

	gmQuaternion m_result;
	float m_trackballSize;

	int m_renormCount;
	int m_lastX, m_lastY;
	int m_winW, m_winH;
};

#endif
